% ------------------------------------------------------------------------%
% This script is used to calculated the R-metric, R-IGD and R-HV in
% particular, for preference-based EMO algorithms. Note that herer we only
% consider the user preference elicited as a aspiration level vector in the
% objective space.
% 
% Author: Dr. Ke Li
% Affliation: CODA Group @ University of Exeter
% Contact: k.li@exeter.ac.uk || https://coda-group.github.io/
%
% Reference: K. Li, K. Deb and X. Yao, "R-Metric: Evaluating the 
% Performance of Preference-Based Evolutionary Multi-Objective Optimization 
% Using Reference Points", IEEE Trans. on Evol. Comput., accepted for
% publication, July 2017.
% ------------------------------------------------------------------------%

close all
clear
clc

format long

%% parameter settings
problems   = 'ZDT1';
refp_index = 1;
objDim     = 2;

problem_id = 1; % id = 1: ZDT1,4; id = 2: ZDT2,6; id = 3: ZDT3; id = 4: DTLZ1; id = 5: DTLZ2-4; id =6: DTLZ5,6; id = 7: DTLZ7
numRun     = 31;
igdsamSize = 10000;

% only useful for the many-objective scenario (i.e., objDim > 3)
no_layers = 2;                  % number of layers
no_gaps   = [3, 2];             % specify the # of divisions on each layer
shrink_factors = [1.0, 0.5];    % shrinkage factor for each layer

%% load path
addpath(['test_data/r-stm/', problems, 'M', num2str(objDim), '_', num2str(refp_index)]);
addpath(['test_data/RNSGA2/', problems, 'M', num2str(objDim), '_', num2str(refp_index)]);

%% aspiration level vector settings
if strcmp(problems, 'ZDT1')
    refp_set = [0.3, 0.4; 0.65, 0.3];
elseif strcmp(problems, 'ZDT2')
    refp_set = [0.2, 0.8; 0.9, 0.4];
elseif strcmp(problems, 'ZDT3')
    refp_set = [0.15, 0.4; 0.4, 0.0];
elseif strcmp(problems, 'ZDT4')
    refp_set = [0.3, 0.4; 0.65, 0.3];
elseif strcmp(problems, 'ZDT6')
    refp_set = [0.9, 0.3; 0.5, 0.7];
elseif strcmp(problems, 'DTLZ1')
    refp_set = [0.05, 0.05, 0.2; 0.3, 0.3, 0.2];
elseif strcmp(problems, 'DTLZ2')
    refp_set = [0.2, 0.5, 0.6; 0.7, 0.8, 0.5];
elseif strcmp(problems, 'DTLZ3')
    refp_set = [0.2, 0.5, 0.6; 0.7, 0.8, 0.5];
elseif strcmp(problems, 'DTLZ4')
    refp_set = [0.2, 0.5, 0.6; 0.7, 0.8, 0.5];
elseif strcmp(problems, 'DTLZ5')
    refp_set = [0.1, 0.3, 0.5; 0.6, 0.7, 0.5];
elseif strcmp(problems, 'DTLZ6')
    refp_set = [0.1, 0.3, 0.5; 0.6, 0.7, 0.5];
elseif strcmp(problems, 'DTLZ7')
    refp_set = [0.165, 0.71, 4.678; 0.75, 0.15, 6.0];
end
ref_point = refp_set(refp_index, :);

%% set worst point
w_point = ref_point + 2 * ones(1, objDim);

%% set trimming radius
radius = 0.2;

%% sample PF points given test problem
[PF, PFsize] = pf_samples(objDim, no_layers, no_gaps, shrink_factors, ...
    igdsamSize, problem_id, radius, ref_point, w_point);

%% initialize data structure (here we use R-NSGA-II, MOEA/D-STM as example)
RNSGA2_IGD = zeros(numRun, 1);
RNSGA2_HV  = zeros(numRun, 1);

STM_IGD = RNSGA2_IGD;
STM_HV = RNSGA2_HV;

for i = 1 : numRun
    %% load data
    STM     = load(['STM_FUN', int2str(i - 1)]);
    RNSGA2  = load(['RNSGA2_FUN', int2str(i - 1)]);
    
    data = [STM; RNSGA2];
    
    %% filter non-dominated data
    STM     = filter_NDS(STM, data);
    RNSGA2  = filter_NDS(RNSGA2, data);
        
    %% preprocess filtered data
    [STM, STM_size]         = preprocessing_asf(STM, ref_point, w_point, radius);
    [RNSGA2, RNSGA2_size]   = preprocessing_asf(RNSGA2, ref_point, w_point, radius);
    
    %% calculate R-IGD and R-HV
    [STM_IGD(i), STM_HV(i)]         = cal_metric(STM, PF, w_point, STM_size, PFsize);
    [RNSGA2_IGD(i), RNSGA2_HV(i)]   = cal_metric(RNSGA2, PF, w_point, RNSGA2_size, PFsize);
end

%% extract effect R-metric values
STM_IGD     = STM_IGD(STM_IGD ~= -1);
RNSGA2_IGD  = RNSGA2_IGD(RNSGA2_IGD ~= -1);

STM_HV     = STM_HV(STM_HV ~= -1);
RNSGA2_HV  = RNSGA2_HV(RNSGA2_HV ~= -1);

%% mean and std of R-IGD and R-HV
mean_IGD_STM = mean(STM_IGD);
std_IGD_STM  = std(STM_IGD);
mean_HV_STM  = mean(STM_HV);
std_HV_STM   = std(STM_HV);

mean_IGD_RNSGA2 = mean(RNSGA2_IGD);
std_IGD_RNSGA2  = std(RNSGA2_IGD);
mean_HV_RNSGA2  = mean(RNSGA2_HV);
std_HV_RNSGA2   = std(RNSGA2_HV);

%% print result
disp(['MOEA/D-STM: R-IGD = ', num2str(mean_IGD_STM), '(', num2str(std_IGD_STM), ')', ', R-HV = ', num2str(mean_HV_STM), '(', num2str(std_HV_STM), ')']);
disp(['R-NSGA2: R-IGD = ', num2str(mean_IGD_RNSGA2), '(', num2str(std_IGD_RNSGA2), ')', ', R-HV = ', num2str(mean_HV_RNSGA2), '(', num2str(std_HV_RNSGA2), ')']);
disp('===================================================================');

%% Wilcoxon rank sum test
higd_array    = -1 * ones(2, 1);
IGD_array     = [mean_IGD_STM; mean_IGD_RNSGA2];
[~, best_idx] = min(IGD_array);
if best_idx == 1
    [~, higd_array(2)] = ranksum(STM_IGD, RNSGA2_IGD);
else
    [~, higd_array(1)] = ranksum(RNSGA2_IGD, STM_IGD);
end
disp(['Wilcoxon rank sum test IGD: ' num2str(higd_array(1)), ', ', num2str(higd_array(2))]);

hhv_array     = -1 * ones(2, 1);
HV_array      = [mean_HV_STM; mean_HV_RNSGA2];
[~, best_idx] = max(HV_array);
if best_idx == 1
    [~, hhv_array(2)] = ranksum(STM_HV, RNSGA2_HV);
else
    [~, hhv_array(1)] = ranksum(RNSGA2_HV, STM_HV);
end
disp(['Wilcoxon rank sum test HV : ' num2str(hhv_array(1)), ', ', num2str(hhv_array(2))]);